/*
 * ============LICENSE_START========================================================================
 * ONAP : ccsdk feature sdnr wt
 * =================================================================================================
 * Copyright (C) 2019 highstreet technologies GmbH Intellectual Property. All rights reserved.
 * Copyright (C) 2021 Samsung Electronics Intellectual Property. All rights reserved.
 * =================================================================================================
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 * ============LICENSE_END==========================================================================
 */

package org.onap.ccsdk.features.sdnr.wt.mountpointregistrar.impl;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import java.io.IOException;
import java.time.Instant;
import java.time.ZoneId;
import java.util.Map;
import org.opendaylight.yang.gen.v1.urn.opendaylight.params.xml.ns.yang.data.provider.rev201110.SeverityType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class DMaaPFaultVESMsgConsumer extends DMaaPVESMsgConsumerImpl {

    private static final Logger LOG = LoggerFactory.getLogger(DMaaPFaultVESMsgConsumer.class);

    public DMaaPFaultVESMsgConsumer(GeneralConfig generalConfig) {
        super(generalConfig);
    }

    @Override
    public void processMsg(String msg) throws Exception {
        String faultNodeId;
        String faultOccurrenceTime;
        String faultObjectId;
        String faultReason;
        String faultSeverity;
        String vesDomain;
        int faultSequence;
        String reportingEntityName;
        ObjectMapper oMapper = new ObjectMapper();
        JsonNode dmaapMessageRootNode;

        LOG.info("Fault VES Message is - {}", msg);
        try {
            dmaapMessageRootNode = oMapper.readTree(msg);
            reportingEntityName = dmaapMessageRootNode.at("/event/commonEventHeader/reportingEntityName").textValue();
            if (reportingEntityName.equals("ONAP SDN-R")) {
                LOG.info(
                        "VES PNF Registration message generated by SDNR, hence no need to process any further; Ignoring the received message");
                return;
            }

            vesDomain = dmaapMessageRootNode.at("/event/commonEventHeader/domain").textValue();
            if (!vesDomain.equalsIgnoreCase("fault")) {
                LOG.warn("Received {} domain VES Message in DMaaP Fault topic, ignoring it", vesDomain);
                return;
            }
            faultNodeId = dmaapMessageRootNode.at("/event/commonEventHeader/sourceName").textValue();
            faultOccurrenceTime = Instant
                    .ofEpochMilli(
                            dmaapMessageRootNode.at("/event/commonEventHeader/startEpochMicrosec").longValue() / 1000)
                    .atZone(ZoneId.of("Z")).toString();
            faultObjectId = dmaapMessageRootNode.at("/event/faultFields/alarmInterfaceA").textValue();
            faultReason = dmaapMessageRootNode.at("/event/faultFields/specificProblem").textValue();
            faultSeverity = dmaapMessageRootNode.at("/event/faultFields/eventSeverity").textValue();
            faultSequence = dmaapMessageRootNode.at("/event/commonEventHeader/sequence").intValue();

            if (faultSeverity.equalsIgnoreCase("critical")) {
                faultSeverity = SeverityType.Critical.toString();
            } else if (faultSeverity.equalsIgnoreCase("major")) {
                faultSeverity = SeverityType.Major.toString();
            } else if (faultSeverity.equalsIgnoreCase("minor")) {
                faultSeverity = SeverityType.Minor.toString();
            } else if (faultSeverity.equalsIgnoreCase("warning")) {
                faultSeverity = SeverityType.Warning.toString();
            } else if (faultSeverity.equalsIgnoreCase("nonalarmed")) {
                faultSeverity = SeverityType.NonAlarmed.toString();
            } else {
                faultSeverity = SeverityType.NonAlarmed.toString();
            }

            String baseUrl = getBaseUrl();
            String sdnrUser = getSDNRUser();
            String sdnrPasswd = getSDNRPasswd();

            Map<String, String> payloadMapMessage = FaultNotificationClient.createFaultNotificationPayloadMap(faultNodeId,
                    Integer.toString(faultSequence), faultOccurrenceTime, faultObjectId, faultReason, faultSeverity);

            FaultNotificationClient faultClient = new FaultNotificationClient(baseUrl);
            LOG.debug("Setting RESTConf Authorization values - {} : {}", sdnrUser, sdnrPasswd);
            faultClient.setAuthorization(sdnrUser, sdnrPasswd);
            String message = faultClient.prepareMessageFromPayloadMap(payloadMapMessage);
            faultClient.sendNotification(message);

        } catch (IOException e) {
            LOG.info("Cannot parse json object ");
            throw new Exception("Cannot parse json object", e);
        }
    }

}
